%{
/***************************************************************************
                          confgrammer.y  -  description
                             -------------------
    copyright            : (C) 2002 by Willy De la Court
    email                : willy.delacourt@pandora.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 ***************************************************************************/


#include <stdio.h>
#include <string.h>
#include <qstringlist.h>
#include "confparser.h"

#define YYSTYPE char *
#define YYDEBUG 1
#define YYMAXDEPTH 0
#define YYPARSE_PARAM conf_parser

extern int yylex(void);
extern int conf_lineno;

void yyerror(const char *str)
{
	fprintf(stderr,"error: %s in line %d\n", str, conf_lineno);
}

QString ip_list;
int global;
dataItem *data = 0L;
subnetItem *currentSubnet = 0L;
hostItem *currentHost = 0L;

void add_range(const char *r1, const char *r2, bool bootp)
{
	data = new rangeItem(r1, r2, bootp);
	if(currentSubnet)
		currentSubnet->addRange((rangeItem *) data);
	//if(bootp)
	//	printf("*range dynamic-bootp %s %s*\n", r1, r2);
	//else
	//	printf("*range %s %s*\n", r1, r2);
}

void add_option_to_tree(void *conf_parser)
{
	if(currentHost)
		currentHost->addOption(data);
	else {
		if (currentSubnet)
			currentSubnet->addOption(data);
		else {
			if(global) {
				//printf("global ");
				ConfParser *p = static_cast<ConfParser *>(conf_parser);
				p->addGlobal(data);
			}
		}
	}
}

void add_option(const char *txt, const char *val, void *conf_parser)
{
	data = new optionItem(txt, val);
	add_option_to_tree(conf_parser);
	//printf("*option %s %s*\n", txt, val);
}

void add_option2(const char *txt, const char *val, void *conf_parser)
{
	data = new dataItem(txt, val);
	add_option_to_tree(conf_parser);
	//printf("*%s %s*\n", txt, val);
}

void add_option_time(const char *txt, const char *val, void *conf_parser)
{
	data = new dataItem(txt, val, "time");
	add_option_to_tree(conf_parser);
	//printf("*%s %s*\n", txt, val);
}


#define ADD_RANGE(r1, r2)		add_range(r1, r2, false);
#define ADD_RANGE2(r1, r2)		add_range(r1, r2, true);
#define ADD_OPTION(var, txt, val) 	add_option(txt, val, conf_parser);
#define ADD_OPTION2(var, txt, val) 	add_option2(txt, val, conf_parser);
#define ADD_OPTION_TIME(var, txt, val) 	add_option_time(txt, val, conf_parser);
#define SET_LIST_SA(var, txt) 		add_option(txt, ip_list, conf_parser);
#define NEW_SCOPE(subnet, netmask) 	currentSubnet = new subnetItem(subnet, netmask); \
					data = currentSubnet; \
					//printf("subnet(%s, %s)\n", subnet, netmask);
#define SAVE_SCOPE() 			ConfParser *p = static_cast<ConfParser *>(conf_parser); \
					p->addSubnet(currentSubnet); \
					currentSubnet = 0L; \
					//printf("saving subnet\n");
#define NEW_HOST(name)			currentHost = new hostItem(name); \
					data = currentHost; \
					//printf("*host %s*\n", name);
#define SAVE_HOST()			if(currentSubnet) { \
						currentSubnet->addHost(currentHost); \
						currentHost = 0L; \
					} \
					//printf("saving host\n");
#define SET_GLOBAL()			global = 1;
#define CLEAR_GLOBAL()			global = 0;
#define SET_COMMENT(val)		if(data) \
						data->setComment(val); \
					//printf("comment %s\n", val);

#ifdef JUNK 
scopeAddresses *sa = 0L;

#define ADD_RANGE(r1, r2)		QString range; range = r1; range += "-"; range += r2; \
						if (sa) sa->include->append(range); \
						//printf("*range %s %s*\n", r1, r2);
#define ADD_OPTION(var, txt, val) 		if(sa) sa->var = val; //printf("*option %s %s*\n", txt, val);
#define ADD_OPTION(var, txt, val) 		QString temp; temp = val; if(sa) sa->var = temp.toInt(); //printf("*option %s %s*\n", txt, val);
#define ADD_OPTION2(var, txt, val) 	QString temp; temp = val; if(sa) sa->var = temp.toInt(); //printf("*%s %s*\n", txt, val);
#define SET_LIST_SA(var, txt) 		if(sa) sa->var = ip_list; //printf("*option %s %s*\n", txt, ip_list.ascii());
#define NEW_SCOPE(subnet, netmask) 	sa = new scopeAddresses(subnet, netmask ); //printf("subnet(%s, %s)\n", subnet, netmask);
#define SAVE_SCOPE() 			if(conf_parser) { DhcpWidget *p = static_cast<DhcpWidget *>(conf_parser); \
						if ( sa ) { p->scopeList->append(sa); sa = 0L; } \
						} //printf("saving subnet\n");
#define SET_GLOBAL()			global = 1; //printf("global ");
#define CLEAR_GLOBAL()			global = 0;

#endif

%}

%token '{'
%token '}'
%token ';'
%token '.'
%token '-'

%token SUBNET
%token NETMASK
%token BYTE
%token WORD
%token OPTION
%token SUBNETMASK
%token ROUTERS
%token NISDOMAIN
%token IPFORWARDING
%token DOMAINNAME
%token DOMAINNAMESERVERS
%token BROADCASTADDRESS
%token NETBIOSNAMESERVERS
%token NETBIOSDDSERVER
%token NETBIOSNODETYPE
%token TIMEOFFSET
%token QUOTE
%token IP
%token RANGE
%token DYNAMICBOOTP
%token DEFAULTLEASETIME
%token MAXLEASETIME
%token HOST
%token NEXTSERVER
%token FIXEDADDRESS
%token HARDWARE
%token ETHERNET
%token NUMBER
%token MAC
%token DOMAIN
%token COMMENT
%%

file:	 /* empty */
	| file all
	;

all:
	  { SET_GLOBAL(); } options comments { CLEAR_GLOBAL() }
	| subnet
	| error ';' 					{ yyclearin; yyerrok; }
	| error '}' 					{ yyclearin; yyerrok; }
	;

subnet:
	SUBNET IP NETMASK IP { NEW_SCOPE($2, $4 ); } comments
	'{' comments
		subnet_parameters
	'}' comments { SAVE_SCOPE(); }
	;

subnet_parameters:
	  /* empty */
	| subnet_parameters options comments
	| subnet_parameters range comments
	| subnet_parameters host comments
	| error ';' 					{ yyclearin; yyerrok; }
	| error '}' 					{ yyclearin; yyerrok; }
	;

range:
	  RANGE DYNAMICBOOTP IP IP ';' 			{ ADD_RANGE2($3, $4); }
	| RANGE IP IP ';' 				{ ADD_RANGE($2, $3); }
	;

options:
	  OPTION SUBNETMASK IP ';'			{ ADD_OPTION(subnetmask, "subnet-mask", $3); }
	| OPTION ROUTERS ip_list ';'			{ SET_LIST_SA(routers, "routers"); }
	| OPTION NETBIOSNAMESERVERS ip_list ';'		{ SET_LIST_SA(netbiosNameServers, "netbios-name-servers"); }
	| OPTION BROADCASTADDRESS IP ';'		{ ADD_OPTION(broadcast, "broadcast-address", $3); }
	| OPTION NETBIOSDDSERVER ip_list ';'		{ SET_LIST_SA(netbiosDDServer, "netbios-dd-server"); }
	| OPTION NETBIOSNODETYPE NUMBER ';'		{ ADD_OPTION(netbiosNodeType, "netbios-node-type", $3); }
	| OPTION DOMAINNAMESERVERS ip_list ';'		{ SET_LIST_SA(nameServers, "domain-name-servers"); }
	| OPTION NISDOMAIN QUOTE DOMAIN QUOTE ';'	{ ADD_OPTION(nisDomain, "nis-domain", $4); }
	| OPTION DOMAINNAME QUOTE DOMAIN QUOTE ';'	{ ADD_OPTION(domainName, "domain-name", $4); }
	| OPTION TIMEOFFSET NUMBER ';'			{ ADD_OPTION(timeOffset, "time-offset", $3); }
	| DEFAULTLEASETIME NUMBER ';'			{ ADD_OPTION_TIME(leaseDurations, "default-lease-time", $2); }
	| MAXLEASETIME NUMBER ';'			{ ADD_OPTION_TIME(maxLeaseDurations, "max-lease-time", $2); }
	;

host:
	HOST WORD { NEW_HOST($2) } comments
	'{' comments
		host_options
	'}' { SAVE_HOST(); }
	;

host_options:
	  /* empty */
	| host_options host_option comments
	| host_options options comments
	| error ';' 					{ yyclearin; yyerrok; }
	| error '}' 					{ yyclearin; yyerrok; }
	;

host_option:
	  FIXEDADDRESS IP ';'				{ ADD_OPTION2(fixedaddress, "fixed-address", $2); }
	| NEXTSERVER DOMAIN ';'				{ ADD_OPTION2(nextserver, "next-server", $2); }
	| HARDWARE ETHERNET MAC ';'			{ ADD_OPTION2(hardware_ethernet, "hardware ethernet", $3); }
	;

ip_list:
	  IP						{ ip_list = $1; }
	| QUOTE DOMAIN QUOTE				{ ip_list = "\""; ip_list += $2; ip_list += "\""; }
	| ip_list ',' IP				{ ip_list += ", "; ip_list += $3; }
	| ip_list ',' QUOTE DOMAIN QUOTE		{ ip_list += ", "; ip_list += "\""; ip_list += $4; ip_list += "\""; }
	;

comments:
	  /* empty */
	| comments COMMENT				{ SET_COMMENT($2); }
	;
%%
