#ifndef _DATACLASSES_
#define _DATACLASSES_

#include <qstring.h>
#include <qstringlist.h>

class dataItem
{
public:
	dataItem(const char *name, const char *data, const char *type = "simple");
	virtual ~dataItem() {};

	void setComment(const char *comment);
	void setData(const char *data);
	virtual QString generate(QString ident);
	virtual QString generateComment(QString ident, bool makeComment = true);

	QString itemName;
	QString itemData;
	QString itemType;
	QStringList itemComments;
};

class optionItem : public  dataItem
{
public:
	optionItem(const char *name, const char *data);
	virtual ~optionItem() {};
	virtual QString generate(QString ident);
};

class rangeItem : public dataItem
{
public:
	rangeItem(const char *ip1, const char *ip2, const char *comment, bool bootp = false);
	rangeItem(const char *ip1, const char *ip2, bool bootp = false);
	virtual ~rangeItem() {};
	virtual QString generate(QString ident);

	QString rangeIP1, rangeIP2;
	bool rangeBootp;
};

class hostItem : public dataItem
{
public:
	hostItem(const char *name);
	virtual ~hostItem() {};
	virtual QString generate(QString ident);
	void addOption(dataItem *option);

	QString itemHostName;
	QList<dataItem> hostOptions;
};


class subnetItem : public dataItem
{
public:
	subnetItem(const char *subnet, const char *netmask);
	virtual ~subnetItem() {};
	virtual QString generate(QString ident);
	void addOption(dataItem *option);
	void addRange(rangeItem *range);
	void addHost(hostItem *host);

	QString itemSubnet;
	QString itemNetmask;
	QList<dataItem> subnetOptions;
	QList<rangeItem> ranges;
	QList<hostItem> hosts;
};

#endif
