/***************************************************************************
                          dhcp_config.cpp  -  description
                             -------------------
    copyright            : (C) 2002 by Willy De la Court
                           Originally based on work done by
                           Copyright (c) 1998-1999 Karl Backstrm

    email                : willy.delacourt@pandora.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 ***************************************************************************/

#include <klistview.h>
#include <qwidgetstack.h>
#include <qlabel.h>
#include <qsplitter.h>
#include <klocale.h>
#include <kconfig.h>
#include <qheader.h>
#include "configwidget.h"
#include "dhcpd_config.h"
#include "server.h"
#include "mlistviewitem.h"
#include "mlistview.h"

DHCPD_Config::DHCPD_Config( QWidget *parent, const char *name ):
	DHCPD_Config_UI(parent, name)
{
	config = new KConfig("kcmdhcpdrc");
	configWidget = 0;
	dummyConfigWidget = new ConfigWidget( configContainer, "configWidget" );
	dummyConfigWidget->description = i18n("No action available");
	connect( configList, SIGNAL(selectionChanged(QListViewItem*)), SLOT(selectionHighlighted(QListViewItem*)));

	addServer *temp = new addServer( config, configList, configContainer, "addServer" );
	connect( temp, SIGNAL(changed(bool)), SLOT(emitChanged(bool)) );
	splitter->setResizeMode( configList, QSplitter::KeepSize );
	configList->setMinimumSize(configList->sizeHint());
	configList->header()->hide();
}

DHCPD_Config::~DHCPD_Config()
{
	// rollback changes so when a user has deleted a server 
	// from the list and pressed cancel no changes are saved
	config->rollback();
	delete config;
}

void DHCPD_Config::load()
{
	config->setGroup("General");
	QStringList serverList = config->readListEntry("Servers");

	for ( QStringList::Iterator it = serverList.begin(); it != serverList.end(); ++it )
	{
		editServer *temp = new editServer(config, configList, *it, configContainer, "editServer");
		connect( temp, SIGNAL(changed(bool)), SLOT(emitChanged(bool)) );
	}
}

void DHCPD_Config::emitChanged(bool state)
{
	emit changed(state);
}

void DHCPD_Config::save()
{
	QWidget *action;
	QListViewItemIterator it( configList );
	for ( ; it.current(); ++it )
	{
		action = ((MListViewItem *) it.current())->actionWidget;
		if ( action && action->className() == "editServer" )
			((editServer *) action)->saveConfig();
	}
}

void DHCPD_Config::restoreDefault()
{
}

void DHCPD_Config::selectionHighlighted(QListViewItem *i)
{
	ConfigWidget *newConfigWidget = 0;
	editServer *tempWidget;
	
	if (!i) return;
	newConfigWidget = ((MListViewItem *)(i))->actionWidget;
	if (!newConfigWidget)
		newConfigWidget = dummyConfigWidget;
	//qDebug(newConfigWidget->className());
	if (QString(newConfigWidget->className()) == "editServer") {
		tempWidget = (editServer *)(newConfigWidget);
		tempWidget->readServerConfig((MListViewItem *) i);
	}
	// open the selected item
	i->setOpen ( true );
	newConfigWidget->setDescription(ActionDescription);
	configContainer->addWidget(newConfigWidget);
	configContainer->raiseWidget(newConfigWidget);
	configContainer->removeWidget(configWidget);
	configWidget = newConfigWidget;
	configContainer->setMinimumSize(configWidget->sizeHint());
	connect( configWidget, SIGNAL(changed(bool)), this, SLOT(emitChanged(bool)) );
}

#include "dhcpd_config.moc"
