/***************************************************************************
                          items.cpp  -  description
                             -------------------
    copyright            : (C) 2002 by Willy De la Court
    email                : willy.delacourt@pandora.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <kconfig.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qtextedit.h>
#include <qwidgetstack.h>
#include <klistview.h>
#include <ktextedit.h>
#include <qstringlist.h>
#include <qregexp.h>
#include <qvalidator.h> 
#include <qlayout.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include "items.h"

oneDataItem::oneDataItem( MListViewItem *listViewItem, QString listViewItemText, dataItem *parserDataItem, QWidget *parent, const char *name )
	: oneItem_UI(parent, name)
{
	thisDataItem = parserDataItem;
	if ( parserDataItem )
	{
		lblOption->setText( thisDataItem->itemName );
		txtValue->setText( thisDataItem->itemData );
		txtComment->setText( thisDataItem->generateComment( "", false ) );
	}
	thisListViewItem = new MListViewItem(listViewItem, listViewItemText, this);
	thisParentList = 0;
	connect( txtValue, SIGNAL( textChanged(const QString&) ), this, SLOT( updateData() ) );
}

oneDataItem::oneDataItem( MListView *listView, QList<dataItem> *parentList, dataItem *parserDataItem, QWidget *parent, const char *name )
	: oneItem_UI(parent, name)
{
	thisParentList = parentList;
	thisDataItem = parserDataItem;
	lblOption->setText( thisDataItem->itemName );
	txtValue->setText( thisDataItem->itemData );
	txtComment->setText( thisDataItem->generateComment( "", false ) );
	thisListViewItem = new MListViewItem(listView, thisDataItem->itemName, thisDataItem->itemData, this);
	connect( txtValue, SIGNAL( textChanged(const QString&) ), this, SLOT( updateData() ) );
	connect( txtComment, SIGNAL( textChanged() ), this, SLOT( updateData() ) );
}

void oneDataItem::updateData()
{
	thisDataItem->itemData = txtValue->text();
	thisDataItem->itemComments = QStringList::split( QRegExp( "\\n" ), txtComment->text() );
	thisListViewItem->setText( 1, thisDataItem->itemData );
}

oneDataItem::~oneDataItem()
{
	thisParentList->remove( thisDataItem );
}

oneDataItemTime::oneDataItemTime( MListView *listView, QList<dataItem> *parentList, dataItem *parserDataItem, QWidget *parent, const char *name )
	: oneItemTime_UI(parent, name)
{
	thisParentList = parentList;
	thisDataItem = parserDataItem;
	lblOption->setText( thisDataItem->itemName );
	long t = thisDataItem->itemData.toLong();
	spnDay->setValue( t / (24 * 60 * 60) );
	spnHour->setValue( ( t % (24 * 60 * 60) ) / ( 60 * 60 ) );
	spnMinute->setValue( ( t % (60 * 60) ) / ( 60 ));
	txtComment->setText( thisDataItem->generateComment( "", false ) );
	thisListViewItem = new MListViewItem(listView, thisDataItem->itemName, thisDataItem->itemData, this);
	connect( spnDay, SIGNAL( valueChanged (int) ), this, SLOT( updateData() ) );
	connect( spnHour, SIGNAL( valueChanged (int) ), this, SLOT( updateData() ) );
	connect( spnMinute, SIGNAL( valueChanged (int) ), this, SLOT( updateData() ) );
	connect( txtComment, SIGNAL( textChanged() ), this, SLOT( updateData() ) );
}

oneDataItemTime::~oneDataItemTime()
{
	thisParentList->remove( thisDataItem );
}

void oneDataItemTime::updateData()
{
	long t;
	t = spnDay->value() * 24 * 60 * 60;
	t += spnHour->value() * 60 * 60;
	t += spnMinute->value() * 60;
	thisDataItem->itemData.setNum( t );
	thisDataItem->itemComments = QStringList::split( QRegExp( "\\n" ), txtComment->text() );
	thisListViewItem->setText( 1, thisDataItem->itemData );
	emit changed( true );
}

rangeDataItem::rangeDataItem( MListView *listViewItem, QList<rangeItem> *parentList, rangeItem *parserDataItem )
{
	thisDataItem = parserDataItem;
	thisParentList = parentList;
	thisListViewItem = new MListViewItem(listViewItem, thisDataItem->rangeIP1, thisDataItem->rangeIP2, thisDataItem->rangeBootp?i18n("Yes"):i18n("No"), this);
}

rangeDataItem::~rangeDataItem()
{
	thisParentList->remove( thisDataItem );
}

rangesDataItem::rangesDataItem( MListViewItem *listViewItem, QString listViewItemText, QList<rangeItem> *parserDataItem, QWidget *parent, const char *name )
	: rangesItem_UI(parent, name)
{
	description = i18n("For any subnet on which addresses will be assigned dynamically,  there  must be at least one range statement.");
	rangeItem *elRange;
	currentListViewItem = 0;
	thisListViewItem = new MListViewItem(listViewItem, listViewItemText, this);
	thisDataItem = parserDataItem;
	for ( elRange = thisDataItem->first(); elRange; elRange = thisDataItem->next() )
		new rangeDataItem(lstView, thisDataItem, elRange);
	
	txtIPFrom->setValidator( new QRegExpValidator( QRegExp( "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+" ), this ) );
	txtIPTo->setValidator( new QRegExpValidator( QRegExp( "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+" ), this ) );

	connect( lstView, SIGNAL(selectionChanged(QListViewItem*)), SLOT(selectionHighlighted(QListViewItem*)) );
	connect( btnDelete, SIGNAL(clicked()), SLOT(remove()) );
	connect( btnNew, SIGNAL(clicked()), SLOT(addNew()) );
	connect( txtIPFrom, SIGNAL( textChanged(const QString&) ), this, SLOT( updateData() ) );
	connect( txtIPTo, SIGNAL( textChanged(const QString&) ), this, SLOT( updateData() ) );
	connect( chkBootp, SIGNAL( toggled(bool) ), this, SLOT( updateData() ) );
	connect( txtComment, SIGNAL( textChanged() ), this, SLOT( updateData() ) );
}

void rangesDataItem::selectionHighlighted(QListViewItem *i)
{
	rangeDataItem *thisRange;
	MListViewItem *thisListViewItem;
	
	// set currentListViewItem to 0 so the updateData 
	// does not get executed when we display the text
	currentListViewItem = 0;
	thisListViewItem = (MListViewItem *)i;
	if ( thisListViewItem )
	{
		thisRange = (rangeDataItem *) (thisListViewItem->actionWidget);
		txtIPFrom->setText( thisRange->thisDataItem->rangeIP1 );
		txtIPTo->setText( thisRange->thisDataItem->rangeIP2 );
		chkBootp->setChecked( thisRange->thisDataItem->rangeBootp );
		txtComment->setText( thisRange->thisDataItem->generateComment( "", false ) );
		currentListViewItem = thisListViewItem;
	}
}

void rangesDataItem::updateData()
{
	rangeDataItem *thisRange;
	
	if ( currentListViewItem )
	{
		thisRange = (rangeDataItem *) (currentListViewItem->actionWidget);
		thisRange->thisDataItem->rangeIP1 = txtIPFrom->text();
		thisRange->thisDataItem->rangeIP2 = txtIPTo->text();
		thisRange->thisDataItem->rangeBootp = chkBootp->isChecked();
		thisRange->thisDataItem->itemComments = QStringList::split( QRegExp( "\\n" ), txtComment->text() );
		currentListViewItem->setText( 0, txtIPFrom->text() );
		currentListViewItem->setText( 1, txtIPTo->text() );
		currentListViewItem->setText( 2, chkBootp->isChecked()?i18n("Yes"):i18n("No") );
		emit changed( true );
	}
}

void rangesDataItem::addNew()
{
	rangeItem *elRange;
	elRange = new rangeItem( txtIPFrom->text(), txtIPTo->text(), txtComment->text(), chkBootp->isChecked() );
	thisDataItem->append( elRange );
	rangeDataItem *newRange = new rangeDataItem( lstView, thisDataItem, elRange );
	lstView->setSelected( newRange->thisListViewItem, true );
	emit changed( true );
}

void rangesDataItem::remove()
{
	rangeDataItem *thisRange;
	
	if ( currentListViewItem )
	{
		thisRange = (rangeDataItem *) (currentListViewItem->actionWidget );
		txtIPFrom->setText( thisRange->thisDataItem->rangeIP1 );
		txtIPTo->setText( thisRange->thisDataItem->rangeIP2 );
		chkBootp->setChecked( thisRange->thisDataItem->rangeBootp );
		txtComment->setText( thisRange->thisDataItem->generateComment( "", false ) );
		delete currentListViewItem;
		currentListViewItem = 0;
		emit changed( true );
	}
}

addSubnetDataItem::addSubnetDataItem( MListViewItem *listViewItem, QString listViewItemText, QList<subnetItem> *parserDataItem, QWidget *parent, const char *name )
	: addSubnetItem_UI(parent, name)
{
	description = i18n("Add a new subnet.\nThe subnet statement is used to provide dhcpd with enough information to tell whether or not an IP address is on that subnet.");
	thisDataItem = parserDataItem;
	thisListViewItem = new MListViewItem(listViewItem, listViewItemText, this);
	txtSubnet->setValidator( new QRegExpValidator( QRegExp( "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+" ), this ) );
	txtNetmask->setValidator( new QRegExpValidator( QRegExp( "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+" ), this ) );
	connect( btnAdd, SIGNAL( clicked() ), SLOT( addSubnet() ) );
}

void addSubnetDataItem::addSubnet()
{
	subnetItem *elSubNet;
	subnetDataItem *thisSubNet;

	elSubNet = new subnetItem(txtSubnet->text(), txtNetmask->text());
	elSubNet->itemComments = QStringList::split( QRegExp( "\\n" ), txtComment->text() );
	thisDataItem->append( elSubNet);
	thisSubNet = new subnetDataItem(thisListViewItem, i18n("Subnet ") + elSubNet->itemName, thisDataItem, elSubNet, 0);
	new rangesDataItem(thisSubNet->thisListViewItem, i18n("Ranges"), &(elSubNet->ranges), 0);
	new optionsItem(thisSubNet->thisListViewItem, i18n("Options"), &(elSubNet->subnetOptions), 0);
	new addHostDataItem(thisSubNet->thisListViewItem, i18n("Hosts"), &(elSubNet->hosts), 0);
	thisSubNet->thisListViewItem->setOpen( true );
	thisSubNet->thisListViewItem->listView()->setSelected( thisSubNet->thisListViewItem, true );

	emit changed( true );
}

addHostDataItem::addHostDataItem( MListViewItem *listViewItem, QString listViewItemText, QList<hostItem> *parserDataItem, QWidget *parent, const char *name )
	: addHostItem_UI(parent, name)
{
	description = i18n("Add a new Host definition.");
	
	thisDataItem = parserDataItem;
	thisListViewItem = new MListViewItem(listViewItem, listViewItemText, this);
	connect( btnAdd, SIGNAL( clicked() ), SLOT( addHost() ) );
}

void addHostDataItem::addHost()
{
	hostItem *elHost;
	hostDataItem *thisHost;
	optionsItem *thisOptions;
	
	elHost = new hostItem( txtHostName->text() );
	elHost->itemComments = QStringList::split( QRegExp( "\\n" ), txtComment->text() );
	thisDataItem->append( elHost );
	thisHost = new hostDataItem(thisListViewItem, txtHostName->text(), thisDataItem, elHost, 0);
	thisOptions = new optionsItem(thisHost->thisListViewItem, i18n("Options"), &(elHost->hostOptions), 0);
	thisOptions->addHostOptions();
	thisHost->thisListViewItem->setOpen( true );
	thisHost->thisListViewItem->listView()->setSelected( thisHost->thisListViewItem, true );

	emit changed( true );
}

hostDataItem::hostDataItem( MListViewItem *listViewItem, QString listViewItemText, QList<hostItem> *parentList, hostItem *parserDataItem, QWidget *parent, const char *name )
	: hostItem_UI(parent, name)
{
	description = i18n("Edit a Host definition.");

	thisParentList = parentList;
	thisDataItem = parserDataItem;
	txtHostName->setText( thisDataItem->itemHostName );
	txtComment->setText( thisDataItem->generateComment( "", false ) );
	thisListViewItem = new MListViewItem(listViewItem, listViewItemText, this);

	connect( txtHostName, SIGNAL( textChanged(const QString&) ), this, SLOT( updateData() ) );
	connect( txtComment, SIGNAL( textChanged() ), this, SLOT( updateData() ) );
	connect( btnDelete, SIGNAL( clicked() ), this, SLOT( remove() ) );
}

void hostDataItem::updateData()
{
	thisDataItem->itemHostName = txtHostName->text();
	thisDataItem->itemComments = QStringList::split( QRegExp( "\\n" ), txtComment->text() );	
	thisListViewItem->setText( 0, thisDataItem->itemHostName );

	emit changed( true );
}

void hostDataItem::remove()
{
	emit changed( true );
	delete thisListViewItem;
}

hostDataItem::~hostDataItem()
{
	thisParentList->remove( thisDataItem );
}

subnetDataItem::subnetDataItem( MListViewItem *listViewItem, QString listViewItemText, QList<subnetItem> *parentList, subnetItem *parserDataItem, QWidget *parent, const char *name )
	: subnetItem_UI(parent, name)
{
	description = i18n("Edit an existing subnet.\nThe subnet statement is used to provide dhcpd with enough information to tell whether or not an IP address is on that subnet.");
	thisParentList = parentList;
	thisDataItem = parserDataItem;
	txtSubnet->setText( thisDataItem->itemSubnet );
	txtNetmask->setText( thisDataItem->itemNetmask );
	txtComment->setText( thisDataItem->generateComment( "", false ) );
	thisListViewItem = new MListViewItem(listViewItem, listViewItemText, this);
	
	txtSubnet->setValidator( new QRegExpValidator( QRegExp( "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+" ), this ) );
	txtNetmask->setValidator( new QRegExpValidator( QRegExp( "[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+" ), this ) );
	
	connect( txtSubnet, SIGNAL( textChanged(const QString&) ), this, SLOT( updateData() ) );
	connect( txtNetmask, SIGNAL( textChanged(const QString&) ), this, SLOT( updateData() ) );
	connect( txtComment, SIGNAL( textChanged() ), this, SLOT( updateData() ) );
	connect( btnDelete, SIGNAL( clicked() ), this, SLOT( remove() ) );
}

void subnetDataItem::updateData()
{
	thisDataItem->itemSubnet = txtSubnet->text();
	thisDataItem->itemNetmask = txtNetmask->text();
	thisDataItem->itemComments = QStringList::split( QRegExp( "\\n" ), txtComment->text() );	
	thisListViewItem->setText( 0, i18n("Subnet ") + thisDataItem->itemSubnet );
	emit changed( true );
}

void subnetDataItem::remove()
{
	emit changed( true );
	delete thisListViewItem;
}

subnetDataItem::~subnetDataItem()
{
	thisParentList->remove( thisDataItem );
}

optionsItem::optionsItem( MListViewItem *listViewItem, QString listViewItemText, QList<dataItem> *parserDataItem, QWidget *parent, const char *name )
	: optionsItem_UI(parent, name)
{
	dataItem *element;

	description = i18n("Manage Options.");
	dummyConfigWidget = new ConfigWidget( configContainer, "configWidget" );
	thisListViewItem = new MListViewItem(listViewItem, listViewItemText, this);
	thisDataItem = parserDataItem;
	currentListViewItem = 0;

	for ( element = thisDataItem->first(); element; element = thisDataItem->next() )
	{
		// TODO Subnet Options: adding more different types like IP IPLIST TIME etc...
		if ( element->itemType == "simple" )
			new oneDataItem(lstOptions, thisDataItem, element, 0);
		else
		if ( element->itemType == "time" )
			new oneDataItemTime(lstOptions, thisDataItem, element, 0);
	}
	const char *globalOptions[] = {
	"broadcast-address",
	"default-lease-time",
	"domain-name",
	"domain-name-servers",
	"ip-forwarding",
	"max-lease-time",
	"netbios-dd-server",
	"netbios-name-servers",
	"netbios-node-type",
	"nis-domain",
	"routers",
	"subnet-mask",
	"time-offset",
	0};

	// TODO check to see if we did not forget any option that is needed
	cmbAllOptions->insertStrList( globalOptions );
	cmbAllOptions->listBox()->sort();

	connect( lstOptions, SIGNAL(selectionChanged(QListViewItem*)), SLOT(selectionHighlighted(QListViewItem*)));
	connect( btnDelete, SIGNAL(clicked()), this, SLOT(remove()) );
	connect( btnAdd, SIGNAL(clicked()), SLOT(add()));
}

void optionsItem::addHostOptions()
{
	const char *hostOptions[] = {
	"fixed-address",
	"hardware ethernet",
	"next-server",
	0};

	cmbAllOptions->insertStrList( hostOptions );
	cmbAllOptions->listBox()->sort();
}

void optionsItem::selectionHighlighted(QListViewItem *i)
{
	ConfigWidget *newConfigWidget = 0;
	
	if ( i )
	{
		currentListViewItem = (MListViewItem *)i;
		newConfigWidget = ((MListViewItem *)(i))->actionWidget;
		if (!newConfigWidget)
			newConfigWidget = dummyConfigWidget;
	}
	else
	{
		newConfigWidget = dummyConfigWidget;
		currentListViewItem = 0;
	}
	//qDebug(newConfigWidget->className());
	configContainer->addWidget(newConfigWidget);
	configContainer->raiseWidget(newConfigWidget);
	configContainer->removeWidget(configWidget);
	configWidget = newConfigWidget;
	configContainer->setMinimumSize( configWidget->sizeHint() );
	// some strange behaviour happening here
	parentWidget()->setMinimumSize( optionsItem_UILayout->sizeHint().expandedTo( configWidget->sizeHint() + QSize(10, 10) ) );
	connect( configWidget, SIGNAL(changed(bool)), this, SLOT(emitChanged(bool)) );
}

void optionsItem::remove()
{
	if ( currentListViewItem )
	{
		delete currentListViewItem;
		currentListViewItem = 0;
		// configWidget was deleted when the currentListViewItem was deleted
		configWidget = 0;
		emit changed( true );
		// set a dummy widget to resolve redraw problems
		selectionHighlighted( 0 );
	}
}

void optionsItem::add()
{
	QString newItem = cmbAllOptions->currentText();
	dataItem *data;
	MListViewItem *newListViewItem;
	
	if ( newItem == "default-lease-time" || newItem == "max-lease-time" )
	{
		data = new dataItem(newItem, "", "time" );
		thisDataItem->append(data);
		oneDataItemTime *newOption = new oneDataItemTime(lstOptions, thisDataItem, data, 0);
		newListViewItem = newOption->thisListViewItem;
	}
	else
	{
		data = new dataItem(newItem, "", "simple" );
		thisDataItem->append(data);
		oneDataItem *newOption = new oneDataItem(lstOptions, thisDataItem, data, 0);
		newListViewItem = newOption->thisListViewItem;
	}
	lstOptions->setSelected( newListViewItem, true );
}

void optionsItem::emitChanged(bool state)
{
	emit changed( state );
}

#include "items.moc"
