/***************************************************************************
                          server.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Willy De la Court
    email                : willy.delacourt@pandora.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 ***************************************************************************/

#include <kurl.h>
#include <kmessagebox.h>
#include "server.h"
#include "confparser.h"
#include "mlistviewitem.h"
 
addServer::addServer( KConfig* kcmdhcp_config, QListView *thisList, QWidget *parent, const char *name )
	: addServer_UI(parent, name)
{
	configListView = thisList;
	config = kcmdhcp_config;
	configGroup = "0";
	description = i18n("Add a new DHCP server to manage");
	allServersItem = new MListViewItem(configListView, i18n("Servers"), this);
	allServersItem->setOpen(true);
	configListView->setSelected(allServersItem, true);

	connect( btnAdd, SIGNAL( clicked() ), SLOT( add() ) );
}

void addServer::add()
{
	editServer *newServer = new editServer(config, configListView, configGroup, 0, "editServer");
	newServer->serverName->setText( serverName->text() );
	newServer->configFileName->setText( configFileName->text().isEmpty()?"/etc/dhcpd.conf":configFileName->text() );
	newServer->leasesFileName->setText( leasesFileName->text().isEmpty()?"/var/lib/dhcp/dhcpd.leases":leasesFileName->text() );
	newServer->serverIsRemote->setChecked( serverIsRemote->isChecked() );
	newServer->remoteHostName->setText( remoteHostName->text().isEmpty()?"localhost":remoteHostName->text() );
	newServer->SSHPort->setText( SSHPort->text().isEmpty()?"22":SSHPort->text() );
	newServer->UserName->setText( UserName->text().isEmpty()?"root":UserName->text() );
	configListView->setSelected( newServer->serverItem, true );
	emit changed( true );
}

editServer::editServer( KConfig *kcmdhcp_config, QListView *thisList, QString serverNumber, QWidget *parent, const char *name )
	: editServer_UI(parent, name)
{
	configListView = thisList;
	allServersItem = configListView->firstChild();
	config = kcmdhcp_config;
	configGroup = serverNumber;
	description = i18n("Edit DHCP server configuration");
	readConfig();
	serverItem = new MListViewItem(allServersItem, serverName->text(), this);
	connect( btnReload, SIGNAL( clicked() ), SLOT( reload() ) );
	connect( btnDelete, SIGNAL( clicked() ), SLOT( deletethis() ) );
	connect( serverName, SIGNAL( textChanged(const QString &) ), SLOT( serverNameChanged(const QString &) ) );
	connect( btnStartServer, SIGNAL( clicked() ), SLOT( actionServerStart() ) );
	connect( btnStopServer, SIGNAL( clicked() ), SLOT( actionServerStop() ) );
	connect( btnRestartServer, SIGNAL( clicked() ), SLOT( actionServerRestart() ) );
	connect( btnReloadServer, SIGNAL( clicked() ), SLOT( actionServerReload() ) );
	Parser = 0;
	configRead = false;
}

void editServer::actionServerStart()   { execute( serverStart->text() );   }
void editServer::actionServerStop()    { execute( serverStop->text() );    }
void editServer::actionServerRestart() { execute( serverRestart->text() ); }
void editServer::actionServerReload()  { execute( serverReload->text() );  }

void editServer::readConfig()
{
	config->setGroup(QString::fromLatin1("Server") + configGroup);

	serverName->setText(config->readEntry("ServerName"));
	configFileName->setText(config->readEntry("Config", "/etc/dhcpd.conf"));
	leasesFileName->setText(config->readEntry("Leases", "/var/lib/dhcp/dhcpd.leases"));
	serverIsRemote->setChecked(config->readBoolEntry("Remote", false));
	remoteHostName->setText(config->readEntry("Hostname", "localhost"));
	SSHPort->setText(config->readEntry("SSHPort", "22"));
	UserName->setText(config->readEntry("UserName", "root"));
	serverStop->setText(config->readEntry("ServerStop", "service dhcpd stop"));
	serverStart->setText(config->readEntry("ServerStart", "service dhcpd start"));
	serverRestart->setText(config->readEntry("ServerRestart", "service dhcpd restart"));
	serverReload->setText(config->readEntry("ServerReload", "service dhcpd reload"));
}

void editServer::saveConfig()
{
	if ( configGroup.toInt() == 0 )
	{
		config->setGroup("General");
		QStringList serverList = config->readListEntry("Servers");

		int val = 0;
		for ( QStringList::Iterator it = serverList.begin(); it != serverList.end(); ++it )
		{
			val = ((*it).toInt() > val)?(*it).toInt():val;
		}
		val++;
		configGroup.setNum( val );
		serverList << configGroup;
		config->writeEntry("Servers", serverList);
		config->sync();
	}
	config->setGroup(QString::fromLatin1("Server") + configGroup);
	config->writeEntry( "ServerName", serverName->text() );
	config->writeEntry( "Config", configFileName->text() );
	config->writeEntry( "Leases", leasesFileName->text() );
	config->writeEntry( "Remote", serverIsRemote->isChecked() );
	config->writeEntry( "HostName", remoteHostName->text() );
	config->writeEntry( "SSHPort", SSHPort->text() );
	config->writeEntry( "UserName", UserName->text() );
	config->writeEntry( "ServerStop", serverStop->text());
	config->writeEntry( "ServerStart", serverStart->text());
	config->writeEntry( "ServerRestart", serverRestart->text());
	config->writeEntry( "ServerReload", serverReload->text());
	config->sync();
	saveServerConfig();
}

void editServer::saveServerConfig()
{
	if ( Parser )
		Parser->save();
}

void editServer::readServerConfig(MListViewItem *thisServerItem)
{
	if ( !configRead )
	{
		Parser = new ConfParser( makeRemoteName( configFileName->text() ), thisServerItem );
		configRead = Parser->Parse();
				
		// open all items under the selected item
		QListViewItem* item = thisServerItem->firstChild();
		for ( ; item ; item = item->itemBelow() )
			item->setOpen( true );
	}
}

void editServer::reload()
{
	if ( configRead )
	{
		while ( serverItem->firstChild() )
		{
			delete serverItem->firstChild();
		}
		delete Parser;
		configRead = false;
	}
	readServerConfig(serverItem);
}

void editServer::deletethis()
{
	if ( configGroup != "0" )
	{
		config->setGroup("General");
		QStringList serverList = config->readListEntry("Servers");
		QStringList newServerList;
		for ( QStringList::Iterator it = serverList.begin(); it != serverList.end(); ++it )
		{
			if ( (*it) != configGroup )
				newServerList << (*it);
		}
		config->writeEntry("Servers", newServerList);
		// Do not sync yet User can still press cancel
	}
	configListView->setSelected( allServersItem, true );
	emit changed( true );
	delete serverItem;
}

void editServer::serverNameChanged(const QString &newName)
{
	serverItem->setText(0, newName);
}

void editServer::execute(QString command)
{
	QString result;
	result = KIO::NetAccess::fish_execute( makeRemoteName( "" ), command, this);
	//qDebug(result.latin1());
	KMessageBox::information(0, result );
}

QString editServer::makeRemoteName(QString fileName)
{
	QString remoteFilename;
	
	if ( serverIsRemote->isChecked() )
	{
		remoteFilename = "fish://";
		remoteFilename += UserName->text() + "@";
		remoteFilename += remoteHostName->text() + ":" + SSHPort->text() ;
		remoteFilename += fileName;
	}
	else
	{
		// using the special fish feature for localhost which uses su to gain root access
		remoteFilename = "fish://";
		remoteFilename += "root@localhost";
		remoteFilename += fileName;
	}
	return remoteFilename;
}

#include "server.moc"
